/* Creates a datagram server.  The port 
   number is passed as an argument.  This
   server runs forever */

#include <sys/types.h>

#ifdef WIN32
#include <winsock.h>
#endif

#ifdef LINUX
#include <arpa/inet.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/select.h>
#include <netdb.h>
#endif

#define JITTER 3  // max allowed jitter

typedef struct {
  char name[16];
  unsigned char *data, datalen;
} button;

unsigned char button_dvd2_data[] = {
  38, 9, 19, 9, 10, 8, 11, 8, 10, 9, 10, 9, 10, 8, 11,
  8, 10, 9, 19, 9, 10, 8, 20, 8, 20, 7, 20, 8, 11, 9, 9,
  9, 20, 8, 10, 9, 10, 8, 20, 8, 11 };

unsigned char button_dvd5_data[] = {
  38, 9, 10, 9, 10, 8, 20, 8, 11, 8, 10, 9, 10, 8, 11, 
  8, 10, 9, 20, 8, 10, 8, 20, 9, 19, 9, 19, 9, 10, 8, 
  11, 8, 20, 8, 10, 9, 10, 8, 20, 8, 11 };

unsigned char button_tv2_data[] = {
  141, 69, 10, 8, 10, 7, 10, 7, 10, 8, 10, 7, 10, 8, 10, 
  25, 10, 8, 10, 25, 10, 25, 10, 25, 10, 25, 10, 25, 10, 
  25, 10, 8, 10, 25, 10, 7, 10, 25, 10, 8, 10, 7, 11, 7, 
  10, 8, 10, 7, 10, 8, 10, 25, 10, 8, 10, 25, 10, 25, 10, 
  25, 10, 25, 10, 25, 10, 25, 10 };

unsigned char button_tvok_data[] = {
  141, 69, 10, 8, 10, 8, 10, 7, 10, 8, 10, 7, 10, 8, 10, 
  25, 10, 8, 10, 25, 10, 25, 10, 25, 10, 25, 10, 25, 10, 
  25, 10, 8, 10, 25, 10, 25, 10, 7, 10, 8, 10, 8, 10, 7, 
  10, 25, 10, 8, 10, 7, 10, 8, 10, 25, 10, 25, 10, 25, 10, 
  26, 9, 8, 10, 25, 10, 25, 10 };

unsigned char button_tvdvdok_data[] = {
  142, 69, 10, 25, 10, 8, 10, 25, 10, 7, 10, 7, 10, 8, 10, 
  25, 10, 8, 9, 8, 10, 25, 10, 8, 10, 25, 10, 25, 10, 
  25, 10, 7, 10, 25, 10, 26, 10, 7, 10, 8, 10, 7, 10, 8, 
  10, 25, 10, 8, 10, 7, 10, 8, 10, 25, 10, 25, 10, 25, 
  10, 25, 10, 8, 10, 25, 10, 25, 10 };

unsigned char button_repeat[] = { 142, 34, 10 };

button buttons[] = {
  { "DVD 2",            button_dvd2_data,    sizeof(button_dvd2_data) },
  { "DVD 5",            button_dvd5_data,    sizeof(button_dvd5_data) },
  { "TV 2",              button_tv2_data,     sizeof(button_tv2_data) },
  { "TV OK",            button_tvok_data,    sizeof(button_tvok_data) },
  { "\007TV/DVD OK", button_tvdvdok_data, sizeof(button_tvdvdok_data) },
  { "Repeat",              button_repeat,       sizeof(button_repeat) },
};


#define CLIENT_APP
#include "../../firmware/ledmatrix.h"

#include "ir.h"

int ir_receive(unsigned char *buf, int n) {
  int nok, i;

  // silently ignore single events
  if(n==1) return 0;

  for(nok=0,i=0;i<sizeof(buttons)/sizeof(button);i++) {
    if(buttons[i].datalen == n) {
      int j, ok = 1;
      
      for(j=0;j<buttons[i].datalen;j++)
	if((buf[j] <= buttons[i].data[j] - JITTER) ||
	   (buf[j] >= buttons[i].data[j] + JITTER)) 
	  ok = 0;
      
      if(ok) {
	nok = 1;
	printf("Match: %s\n", buttons[i].name);
      }
    }
  }
  
  if(!nok) {
    printf("Unknown %d bytes\n", n);
    for(i=0;i<n;i++) {
      printf("%d, ", buf[i]);
    }
    printf("\n");
  }

  return 0;
}
