#include <stdio.h>
#include <stdlib.h>

#define CLIENT_APP
#include "../../firmware/ledmatrix.h"
#include "image.h"

static int get_pixel(FILE *file) {
  static int blue_err, red_err, green_err;
  int pixel = 0;

  if(file) {
    int r = fgetc(file);
    int g = fgetc(file);
    int b = fgetc(file);
    if(r > 10) {
      if(r < 245) red_err++;
      else pixel |= 1;
    }
    if(g > 10) {
      if(g < 245) green_err++;
      else pixel |= 2;
    }
    if(b > 10) blue_err++;

    return pixel;
  }

  printf("Pixel errors: red %d, green %d, blue %d\n", 
	 red_err, green_err, blue_err);

  return 0;
}

image_t *rgb_read(char *filename) {
  FILE *file;
  int flen, i;
  image_t *image = NULL;

  if((file = fopen(filename, "rb")) == NULL) {
    perror("open(infile)");
    return NULL;
  }

  // determine input file size
  fseek(file, 0l, SEEK_END);
  flen = ftell(file);
  fseek(file, 0l, SEEK_SET);

  // check input file size
  if(flen != 3 * DSP_W * DSP_PIX_X * DSP_H * DSP_PIX_Y) {
    printf("unexpected image size %d, expected 3*%d*%d\n", flen, 
	   DSP_W * DSP_PIX_X, DSP_H * DSP_PIX_Y);

    fclose(file);
    return NULL;
  }

  image = malloc(sizeof(image_t));
  image->width = DSP_W * DSP_PIX_X;
  image->height = DSP_H * DSP_PIX_Y;
  image->data = malloc(image->width * image->height / 4);
  
  // load image pixel by pixel
  for(i=0;i<DSP_MEM;i++) {
    int j, byte = 0;

    // four pixel make one byte
    for(j=0;j<4;j++) {
      byte <<= 2;
      byte |= get_pixel(file);
    }

    image->data[i] = byte;
  }

  fclose(file);

  // close input file and display pixel statistics
  get_pixel(NULL);
  
  return image;
}

